@@ -22,6 +22,7 @@
 #include "lib/util.h"
 #include "lib/vswitch-idl.h"
 #include "openvswitch/vlog.h"
+#include "socket-util.h"
 
 /* OVN includes. */
 #include "encaps.h"
@@ -447,6 +448,7 @@ local_nonvif_data_run(const struct ovsrec_bridge *br_int,
                 tun->chassis_id = xstrdup(tunnel_id);
                 tun->ofport = u16_to_ofp(ofport);
                 tun->type = tunnel_type;
+                tun->is_ipv6 = ip ? addr_is_ipv6(ip) : false;
 
                 free(hash_id);
                 free(ip);
@@ -645,3 +647,28 @@ datapath_is_transit_switch(const struct sbrec_datapath_binding *ldp)
 {
     return smap_get(&ldp->external_ids, "interconn-ts") != NULL;
 }
+
+bool
+lb_is_local(const struct sbrec_load_balancer *sbrec_lb,
+            const struct hmap *local_datapaths)
+{
+    /* Check if the lb is local or not.  It is enough to find one datapath
+     * in "local_datapaths" to consider the LB to be local. */
+    for (size_t i = 0; i < sbrec_lb->n_datapaths; i++) {
+        if (get_local_datapath(local_datapaths,
+                               sbrec_lb->datapaths[i]->tunnel_key)) {
+            return true;
+        }
+    }
+
+    struct sbrec_logical_dp_group *dp_group = sbrec_lb->datapath_group;
+
+    for (size_t i = 0; dp_group && i < dp_group->n_datapaths; i++) {
+        if (get_local_datapath(local_datapaths,
+                               dp_group->datapaths[i]->tunnel_key)) {
+            return true;
+        }
+    }
+
+    return false;
+}
