@@ -43,8 +43,15 @@ const struct sbrec_port_binding *lport_lookup_by_key(
     struct ovsdb_idl_index *sbrec_port_binding_by_key,
     uint64_t dp_key, uint64_t port_key);
 
-bool lport_can_bind_on_this_chassis(const struct sbrec_chassis *chassis_rec,
-                                   const struct sbrec_port_binding *pb);
+enum can_bind {
+    CANNOT_BIND = 0,
+    CAN_BIND_AS_MAIN,
+    CAN_BIND_AS_ADDITIONAL,
+};
+
+enum can_bind
+lport_can_bind_on_this_chassis(const struct sbrec_chassis *chassis_rec,
+                               const struct sbrec_port_binding *pb);
 
 const struct sbrec_datapath_binding *datapath_lookup_by_key(
     struct ovsdb_idl_index *sbrec_datapath_binding_by_key, uint64_t dp_key);
@@ -63,4 +70,7 @@ const struct sbrec_port_binding *lport_get_peer(
 const struct sbrec_port_binding *lport_get_l3gw_peer(
     const struct sbrec_port_binding *,
     struct ovsdb_idl_index *sbrec_port_binding_by_name);
+bool
+lport_is_activated_by_activation_strategy(const struct sbrec_port_binding *pb,
+                                          const struct sbrec_chassis *chassis);
 #endif /* controller/lport.h */
