@@ -22,6 +22,7 @@
 #include "openvswitch/meta-flow.h"
 #include "ovsdb-idl.h"
 #include "hindex.h"
+#include "lib/uuidset.h"
 
 struct ovn_extend_table;
 struct hmap;
@@ -57,6 +58,7 @@ enum mf_field_id ofctrl_get_mf_field_id(void);
 void ofctrl_put(struct ovn_desired_flow_table *lflow_table,
                 struct ovn_desired_flow_table *pflow_table,
                 struct shash *pending_ct_zones,
+                struct hmap *pending_lb_tuples,
                 const struct sbrec_meter_table *,
                 uint64_t nb_cfg,
                 bool lflow_changed,
@@ -70,7 +72,8 @@ void ofctrl_ct_flush_zone(uint16_t zone_id);
 
 char *ofctrl_inject_pkt(const struct ovsrec_bridge *br_int,
                         const char *flow_s, const struct shash *addr_sets,
-                        const struct shash *port_groups);
+                        const struct shash *port_groups,
+                        const struct smap *template_vars);
 
 /* Flow table interfaces to the rest of ovn-controller. */
 
@@ -118,14 +121,8 @@ void ofctrl_remove_flows(struct ovn_desired_flow_table *,
  *
  * It adds all the sb_uuids that are actually removed in the
  * flood_remove_nodes. */
-struct ofctrl_flood_remove_node {
-    struct hmap_node hmap_node;
-    struct uuid sb_uuid;
-};
 void ofctrl_flood_remove_flows(struct ovn_desired_flow_table *,
-                               struct hmap *flood_remove_nodes);
-void ofctrl_flood_remove_add_node(struct hmap *flood_remove_nodes,
-                                  const struct uuid *sb_uuid);
+                               struct uuidset *flood_remove_nodes);
 bool ofctrl_remove_flows_for_as_ip(struct ovn_desired_flow_table *,
                                    const struct uuid *lflow_uuid,
                                    const struct addrset_info *,
