@@ -15,6 +15,7 @@
 
 #include <config.h>
 
+#include "chassis.h"
 #include "patch.h"
 #include "ovsport.h"
 
@@ -131,7 +132,10 @@ add_ovs_bridge_mappings(const struct ovsrec_open_vswitch_table *ovs_table,
         const char *mappings_cfg;
         char *cur, *next, *start;
 
-        mappings_cfg = smap_get(&cfg->external_ids, "ovn-bridge-mappings");
+        const char *chassis_id = get_ovs_chassis_id(ovs_table);
+        mappings_cfg =
+            get_chassis_external_id_value(
+                &cfg->external_ids, chassis_id, "ovn-bridge-mappings", NULL);
         if (!mappings_cfg || !mappings_cfg[0]) {
             return;
         }
@@ -267,12 +271,26 @@ add_bridge_mappings(struct ovsdb_idl_txn *ovs_idl_txn,
     shash_destroy(&bridge_mappings);
 }
 
+static const struct ovsrec_port *
+get_port(struct ovsdb_idl_index *ovsrec_port_by_name, const char *name)
+{
+    struct ovsrec_port *target =
+        ovsrec_port_index_init_row(ovsrec_port_by_name);
+    ovsrec_port_index_set_name(target, name);
+
+    const struct ovsrec_port *port = NULL;
+    OVSREC_PORT_FOR_EACH_EQUAL (port, target, ovsrec_port_by_name) {
+    }
+    ovsrec_port_index_destroy_row(target);
+    return port;
+}
+
 void
 patch_run(struct ovsdb_idl_txn *ovs_idl_txn,
           struct ovsdb_idl_index *sbrec_port_binding_by_type,
           const struct ovsrec_bridge_table *bridge_table,
           const struct ovsrec_open_vswitch_table *ovs_table,
-          const struct ovsrec_port_table *port_table,
+          struct ovsdb_idl_index *ovsrec_port_by_name,
           const struct ovsrec_bridge *br_int,
           const struct sbrec_chassis *chassis,
           const struct hmap *local_datapaths)
@@ -289,7 +307,8 @@ patch_run(struct ovsdb_idl_txn *ovs_idl_txn,
      * leaving useless ports on upgrade. */
     struct shash existing_ports = SHASH_INITIALIZER(&existing_ports);
     const struct ovsrec_port *port;
-    OVSREC_PORT_TABLE_FOR_EACH (port, port_table) {
+    for (size_t i = 0; i < br_int->n_ports; i++) {
+        port = br_int->ports[i];
         if (smap_get(&port->external_ids, "ovn-localnet-port")
             || smap_get(&port->external_ids, "ovn-l2gateway-port")
             || smap_get(&port->external_ids, "ovn-l3gateway-port")
@@ -307,11 +326,35 @@ patch_run(struct ovsdb_idl_txn *ovs_idl_txn,
 
     /* Now 'existing_ports' only still contains patch ports that exist in the
      * database but shouldn't.  Delete them from the database. */
-    struct shash_node *port_node, *port_next_node;
-    SHASH_FOR_EACH_SAFE (port_node, port_next_node, &existing_ports) {
+    struct shash_node *port_node;
+    SHASH_FOR_EACH_SAFE (port_node, &existing_ports) {
         port = port_node->data;
         shash_delete(&existing_ports, port_node);
-        remove_port(bridge_table, port);
+        /* Wait for some iterations before really deleting any patch ports,
+         * because with conditional monitoring it is possible that related SB
+         * data is not completely downloaded yet after last restart of
+         * ovn-controller.  Otherwise it may cause unncessary dataplane
+         * interruption during restart/upgrade. */
+        if (!daemon_started_recently()) {
+            /* delete peer patch port first */
+            for (size_t i = 0; i < port->n_interfaces; i++) {
+                struct ovsrec_interface *iface = port->interfaces[i];
+                if (strcmp(iface->type, "patch")) {
+                    continue;
+                }
+                const char *iface_peer = smap_get(&iface->options, "peer");
+                if (iface_peer) {
+                    const struct ovsrec_port *peer_port =
+                        get_port(ovsrec_port_by_name, iface_peer);
+                    if (peer_port) {
+                        remove_port(bridge_table, peer_port);
+                    }
+                }
+            }
+
+            /* now delete integration bridge patch port */
+            remove_port(bridge_table, port);
+        }
     }
     shash_destroy(&existing_ports);
 }
