@@ -130,9 +130,6 @@ ovn_init_symtab(struct shash *symtab)
     expr_symtab_add_subfield(symtab, "flags.use_snat_zone", NULL,
                              flags_str);
 
-    snprintf(flags_str, sizeof flags_str, "flags[%d]", MLF_FROM_CTRL_BIT);
-    expr_symtab_add_subfield(symtab, "flags.from_ctrl", NULL, flags_str);
-
     /* Connection tracking state. */
     expr_symtab_add_field_scoped(symtab, "ct_mark", MFF_CT_MARK, NULL, false,
                                  WR_CT_COMMIT);
@@ -284,6 +281,14 @@ ovn_init_symtab(struct shash *symtab)
     expr_symtab_add_field(symtab, "arp.tpa", MFF_ARP_TPA, "arp", false);
     expr_symtab_add_field(symtab, "arp.tha", MFF_ARP_THA, "arp", false);
 
+    /* RARPs use the same layout as arp packets -> use the same field_id */
+    expr_symtab_add_predicate(symtab, "rarp", "eth.type == 0x8035");
+    expr_symtab_add_field(symtab, "rarp.op", MFF_ARP_OP, "rarp", false);
+    expr_symtab_add_field(symtab, "rarp.spa", MFF_ARP_SPA, "rarp", false);
+    expr_symtab_add_field(symtab, "rarp.sha", MFF_ARP_SHA, "rarp", false);
+    expr_symtab_add_field(symtab, "rarp.tpa", MFF_ARP_TPA, "rarp", false);
+    expr_symtab_add_field(symtab, "rarp.tha", MFF_ARP_THA, "rarp", false);
+
     expr_symtab_add_predicate(symtab, "nd",
               "icmp6.type == {135, 136} && icmp6.code == 0 && ip.ttl == 255");
     expr_symtab_add_predicate(symtab, "nd_ns",
