@@ -37,11 +37,11 @@ struct bfd_msg {
     uint8_t flags;
     uint8_t mult;
     uint8_t length;
-    ovs_be32 my_disc;
-    ovs_be32 your_disc;
-    ovs_be32 min_tx;
-    ovs_be32 min_rx;
-    ovs_be32 min_rx_echo;
+    ovs_16aligned_be32 my_disc;
+    ovs_16aligned_be32 your_disc;
+    ovs_16aligned_be32 min_tx;
+    ovs_16aligned_be32 min_rx;
+    ovs_16aligned_be32 min_rx_echo;
 };
 BUILD_ASSERT_DECL(BFD_PACKET_LEN == sizeof(struct bfd_msg));
 
@@ -202,7 +202,7 @@ dhcp_opt_add(struct hmap *dhcp_opts, char *opt_name, size_t code, char *type)
 }
 
 static inline void
-gen_opts_destroy(struct hmap *gen_opts)
+gen_opts_clear(struct hmap *gen_opts)
 {
     struct gen_opts_map *gen_opt;
     HMAP_FOR_EACH_POP (gen_opt, hmap_node, gen_opts) {
@@ -210,6 +210,12 @@ gen_opts_destroy(struct hmap *gen_opts)
         free(gen_opt->type);
         free(gen_opt);
     }
+}
+
+static inline void
+gen_opts_destroy(struct hmap *gen_opts)
+{
+    gen_opts_clear(gen_opts);
     hmap_destroy(gen_opts);
 }
 
@@ -219,6 +225,12 @@ dhcp_opts_destroy(struct hmap *dhcp_opts)
     gen_opts_destroy(dhcp_opts);
 }
 
+static inline void
+dhcp_opts_clear(struct hmap *dhcp_opts)
+{
+    gen_opts_clear(dhcp_opts);
+}
+
 OVS_PACKED(
 struct dhcp_opt_header {
     uint8_t code;
@@ -228,12 +240,6 @@ struct dhcp_opt_header {
 #define DHCP_OPT_PAYLOAD(hdr) \
     (void *)((char *)hdr + sizeof(struct dhcp_opt_header))
 
-/* Used in the OpenFlow PACKET_IN userdata */
-struct dhcp_opt6_header {
-    ovs_be16 opt_code;
-    ovs_be16 size;
-};
-
 /* These are not defined in ovs/lib/dhcp.h, hence defining here. */
 #define OVN_DHCP_MSG_DECLINE        4
 #define OVN_DHCP_MSG_RELEASE        7
@@ -412,6 +418,8 @@ nd_ra_opts_destroy(struct hmap *nd_ra_opts)
 static inline void
 nd_ra_opts_init(struct hmap *nd_ra_opts)
 {
+    hmap_init(nd_ra_opts);
+
     nd_ra_opt_add(nd_ra_opts, "addr_mode", ND_RA_FLAG_ADDR_MODE, "str");
     nd_ra_opt_add(nd_ra_opts, "router_preference", ND_RA_FLAG_PRF, "str");
     nd_ra_opt_add(nd_ra_opts, "slla", ND_OPT_SOURCE_LINKADDR, "mac");
