@@ -33,6 +33,7 @@
 #include "lib/ovn-l7.h"
 #include "lib/ovn-nb-idl.h"
 #include "lib/ovn-sb-idl.h"
+#include "lib/ovs-rcu.h"
 #include "openvswitch/poll-loop.h"
 #include "simap.h"
 #include "stopwatch.h"
@@ -519,7 +520,9 @@ update_sequence_numbers(int64_t loop_start_time,
                              chassis_priv->name);
             }
 
-            if (chassis_priv->nb_cfg < hv_cfg) {
+            /* Detect if overflows happened within the cfg update. */
+            int64_t delta = chassis_priv->nb_cfg - hv_cfg;
+            if (chassis_priv->nb_cfg < hv_cfg || delta > INT32_MAX) {
                 hv_cfg = chassis_priv->nb_cfg;
                 hv_cfg_ts = chassis_priv->nb_cfg_timestamp;
             } else if (chassis_priv->nb_cfg == hv_cfg &&
@@ -1048,6 +1051,8 @@ main(int argc, char *argv[])
     ovsdb_idl_loop_destroy(&ovnnb_idl_loop);
     ovsdb_idl_loop_destroy(&ovnsb_idl_loop);
     service_stop();
+    run_update_worker_pool(0);
+    ovsrcu_exit();
 
     exit(res);
 }
