@@ -199,6 +199,8 @@ ovn_dbctl_main(int argc, char *argv[],
     idl = the_idl = ovsdb_idl_create_unconnected(dbctl_options->idl_class,
                                                  daemon_mode);
     ovsdb_idl_set_shuffle_remotes(idl, shuffle_remotes);
+    /* "set_db_change_aware" is true iff in daemon mode. */
+    ovsdb_idl_set_db_change_aware(idl, daemon_mode);
     /* "retry" is true iff in daemon mode. */
     ovsdb_idl_set_remote(idl, db, daemon_mode);
     ovsdb_idl_set_leader_only(idl, leader_only);
@@ -723,7 +725,7 @@ do_dbctl(const struct ovn_dbctl_options *dbctl_options,
     ctl_context_init(ctx, NULL, idl, txn, symtab, NULL);
     for (size_t i = 0; i < n_commands; i++) {
         struct ctl_command *c = &commands[i];
-        ctl_context_init_command(ctx, c);
+        ctl_context_init_command(ctx, c, c == &commands[n_commands - 1]);
         if (c->syntax->run) {
             (c->syntax->run)(ctx);
         }
